@@ -1,8 +1,8 @@
 /* ldapmodify.c - generic program to modify or add entries using LDAP */
-/* $OpenLDAP: pkg/ldap/clients/tools/ldapmodify.c,v 1.186.2.3 2007/08/31 23:13:50 quanah Exp $ */
+/* $OpenLDAP: pkg/ldap/clients/tools/ldapmodify.c,v 1.186.2.11 2009/07/08 00:28:21 quanah Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2007 The OpenLDAP Foundation.
+ * Copyright 1998-2009 The OpenLDAP Foundation.
  * Portions Copyright 2006 Howard Chu.
  * Portions Copyright 1998-2003 Kurt D. Zeilenga.
  * Portions Copyright 1998-2001 Net Boolean Incorporated.
@@ -70,15 +70,14 @@
 
 #include "common.h"
 
-static int	ldapadd, force = 0;
+static int	ldapadd;
 static char *rejfile = NULL;
 static LDAP	*ld = NULL;
 
 #define	M_SEP	0x7f
 
-/* strings found in replog/LDIF entries (mostly lifted from slurpd/slurp.h) */
+/* strings found in LDIF entries */
 static struct berval BV_VERSION = BER_BVC("version");
-static struct berval BV_REPLICA = BER_BVC("replica");
 static struct berval BV_DN = BER_BVC("dn");
 static struct berval BV_CONTROL = BER_BVC("control");
 static struct berval BV_CHANGETYPE = BER_BVC("changetype");
@@ -96,8 +95,8 @@ static struct berval BV_NEWRDN = BER_BVC("newrdn");
 static struct berval BV_DELETEOLDRDN = BER_BVC("deleteoldrdn");
 static struct berval BV_NEWSUP = BER_BVC("newsuperior");
 
-#define	BVICMP(a,b)	((a)->bv_len != (b)->bv_len ? \
-	(a)->bv_len - (b)->bv_len : strcasecmp((a)->bv_val, (b)->bv_val))
+#define	BV_CASEMATCH(a, b) \
+	((a)->bv_len == (b)->bv_len && 0 == strcasecmp((a)->bv_val, (b)->bv_val))
 
 static int process_ldif_rec LDAP_P(( char *rbuf, int lineno ));
 static int parse_ldif_control LDAP_P(( struct berval *val, LDAPControl ***pctrls ));
@@ -144,7 +143,6 @@ usage( void )
  	fprintf( stderr,
 		_("             [!]txn=<commit|abort>         (transaction)\n"));
 #endif
-	fprintf( stderr, _("  -F         force all changes records to be used\n"));
 	fprintf( stderr, _("  -S file    write skipped modifications to `file'\n"));
 
 	tool_common_usage();
@@ -152,8 +150,8 @@ usage( void )
 }
 
 
-const char options[] = "aE:FrS:"
-	"cd:D:e:f:h:H:IMnO:o:p:P:QR:U:vVw:WxX:y:Y:Z";
+const char options[] = "aE:rS:"
+	"cd:D:e:f:h:H:IMnNO:o:p:P:QR:U:vVw:WxX:y:Y:Z";
 
 int
 handle_private_option( int i )
@@ -217,10 +215,6 @@ handle_private_option( int i )
 		ldapadd = 1;
 		break;
 
-	case 'F':	/* force all changes records to be used */
-		force = 1;
-		break;
-
 	case 'r':	/* replace (obsolete) */
 		break;
 
@@ -246,7 +240,7 @@ main( int argc, char **argv )
 	FILE		*rejfp;
 	struct LDIFFP *ldiffp, ldifdummy = {0};
 	char		*matched_msg, *error_msg;
-	int		rc, retval;
+	int		rc, retval, ldifrc;
 	int		len;
 	int		i = 0;
 	int		lineno, nextline = 0, lmax = 0;
@@ -332,8 +326,8 @@ main( int argc, char **argv )
 	rc = 0;
 	retval = 0;
 	lineno = 1;
-	while (( rc == 0 || contoper ) && ldif_read_record( ldiffp, &nextline,
-		&rbuf, &lmax ))
+	while (( rc == 0 || contoper ) && ( ldifrc = ldif_read_record( ldiffp, &nextline,
+		&rbuf, &lmax )) > 0 )
 	{
 		if ( rejfp ) {
 			len = strlen( rbuf );
@@ -375,6 +369,9 @@ main( int argc, char **argv )
 	}
 	ber_memfree( rbuf );
 
+	if ( ldifrc < 0 )
+		retval = LDAP_OTHER;
+
 #ifdef LDAP_X_TXN
 	if( retval == 0 && txn ) {
 		rc = ldap_set_option( ld, LDAP_OPT_SERVER_CONTROLS, NULL );
@@ -408,10 +405,10 @@ static int
 process_ldif_rec( char *rbuf, int linenum )
 {
 	char	*line, *dn, *newrdn, *newsup;
-	int		rc, modop, replicaport;
+	int		rc, modop;
 	int		expect_modop, expect_sep;
 	int		deleteoldrdn;
-	int		saw_replica, use_record, new_entry, delete_entry, got_all;
+	int		new_entry, delete_entry, got_all;
 	LDAPMod	**pmods, *lm = NULL;
 	int version;
 	LDAPControl **pctrls;
@@ -422,11 +419,10 @@ process_ldif_rec( char *rbuf, int linenum )
 
 	new_entry = ldapadd;
 
-	rc = got_all = saw_replica = delete_entry = modop = expect_modop = 0;
+	rc = got_all = delete_entry = modop = expect_modop = 0;
 	expect_sep = 0;
 	version = 0;
 	deleteoldrdn = 1;
-	use_record = force;
 	pmods = NULL;
 	pctrls = NULL;
 	dn = newrdn = newsup = NULL;
@@ -464,27 +460,7 @@ process_ldif_rec( char *rbuf, int linenum )
 		freeval[i] = freev;
 
 		if ( dn == NULL ) {
-			if ( !use_record && !BVICMP( btype+i, &BV_REPLICA )) {
-				char *p;
-				++saw_replica;
-				if (( p = strchr( vals[i].bv_val, ':' )) == NULL ) {
-					replicaport = 0;
-				} else {
-					*p++ = '\0';
-					if ( lutil_atoi( &replicaport, p ) != 0 ) {
-						fprintf( stderr, _("%s: unable to parse replica port \"%s\" (line %d) entry: \"%s\"\n"),
-							prog, p, linenum+i, dn == NULL ? "" : dn );
-						rc = LDAP_PARAM_ERROR;
-						break;
-					}
-				}
-				if ( ldaphost != NULL &&
-					strcasecmp( vals[i].bv_val, ldaphost ) == 0 &&
-					replicaport == ldapport )
-				{
-					use_record = 1;
-				}
-			} else if ( linenum+i == 1 && !BVICMP( btype+i, &BV_VERSION )) {
+			if ( linenum+i == 1 && BV_CASEMATCH( btype+i, &BV_VERSION )) {
 				int	v;
 				if( vals[i].bv_len == 0 || lutil_atoi( &v, vals[i].bv_val) != 0 || v != 1 ) {
 					fprintf( stderr,
@@ -493,16 +469,9 @@ process_ldif_rec( char *rbuf, int linenum )
 				}
 				version++;
 
-			} else if ( !BVICMP( btype+i, &BV_DN )) {
+			} else if ( BV_CASEMATCH( btype+i, &BV_DN )) {
 				dn = vals[i].bv_val;
 				idn = i;
-				if ( !use_record && saw_replica ) {
-					printf(_("%s: skipping change record for entry: %s at line %d\n"),
-						prog, dn, linenum+i);
-					printf(_("\t(LDAP host/port does not match replica: lines)\n"));
-					rc = 0;
-					goto leave;
-				}
 			}
 			/* skip all lines until we see "dn:" */
 		}
@@ -528,7 +497,7 @@ process_ldif_rec( char *rbuf, int linenum )
 
 	i = idn+1;
 	/* Check for "control" tag after dn and before changetype. */
-	if (!BVICMP( btype+i, &BV_CONTROL)) {
+	if ( BV_CASEMATCH( btype+i, &BV_CONTROL )) {
 		/* Parse and add it to the list of controls */
 		rc = parse_ldif_control( vals+i, &pctrls );
 		if (rc != 0) {
@@ -549,7 +518,7 @@ short_input:
 	}
 
 	/* Check for changetype */
-	if ( !BVICMP( btype+i, &BV_CHANGETYPE )) {
+	if ( BV_CASEMATCH( btype+i, &BV_CHANGETYPE )) {
 #ifdef LIBERAL_CHANGETYPE_MODOP
 		/* trim trailing spaces (and log warning ...) */
 		int icnt;
@@ -567,20 +536,20 @@ short_input:
 		}
 #endif /* LIBERAL_CHANGETYPE_MODOP */
 
-		if ( BVICMP( vals+i, &BV_MODIFYCT ) == 0 ) {
+		if ( BV_CASEMATCH( vals+i, &BV_MODIFYCT )) {
 			new_entry = 0;
 			expect_modop = 1;
-		} else if ( BVICMP( vals+i, &BV_ADDCT ) == 0 ) {
+		} else if ( BV_CASEMATCH( vals+i, &BV_ADDCT )) {
 			new_entry = 1;
 			modop = LDAP_MOD_ADD;
-		} else if ( BVICMP( vals+i, &BV_MODRDNCT ) == 0
-			|| BVICMP( vals+i, &BV_MODDNCT ) == 0
-			|| BVICMP( vals+i, &BV_RENAMECT ) == 0)
+		} else if ( BV_CASEMATCH( vals+i, &BV_MODRDNCT )
+			|| BV_CASEMATCH( vals+i, &BV_MODDNCT )
+			|| BV_CASEMATCH( vals+i, &BV_RENAMECT ))
 		{
 			i++;
 			if ( i >= lines )
 				goto short_input;
-			if ( BVICMP( btype+i, &BV_NEWRDN )) {
+			if ( !BV_CASEMATCH( btype+i, &BV_NEWRDN )) {
 				fprintf( stderr, _("%s: expecting \"%s:\" but saw"
 					" \"%s:\" (line %d, entry \"%s\")\n"),
 					prog, BV_NEWRDN.bv_val, btype[i].bv_val, linenum+i, dn );
@@ -591,7 +560,7 @@ short_input:
 			i++;
 			if ( i >= lines )
 				goto short_input;
-			if ( BVICMP( btype+i, &BV_DELETEOLDRDN )) {
+			if ( !BV_CASEMATCH( btype+i, &BV_DELETEOLDRDN )) {
 				fprintf( stderr, _("%s: expecting \"%s:\" but saw"
 					" \"%s:\" (line %d, entry \"%s\")\n"),
 					prog, BV_DELETEOLDRDN.bv_val, btype[i].bv_val, linenum+i, dn );
@@ -601,7 +570,7 @@ short_input:
 			deleteoldrdn = ( vals[i].bv_val[0] == '0' ) ? 0 : 1;
 			i++;
 			if ( i < lines ) {
-				if ( BVICMP( btype+i, &BV_NEWSUP )) {
+				if ( !BV_CASEMATCH( btype+i, &BV_NEWSUP )) {
 					fprintf( stderr, _("%s: expecting \"%s:\" but saw"
 						" \"%s:\" (line %d, entry \"%s\")\n"),
 						prog, BV_NEWSUP.bv_val, btype[i].bv_val, linenum+i, dn );
@@ -612,7 +581,7 @@ short_input:
 				i++;
 			}
 			got_all = 1;
-		} else if ( BVICMP( vals+i, &BV_DELETECT ) == 0 ) {
+		} else if ( BV_CASEMATCH( vals+i, &BV_DELETECT )) {
 			got_all = delete_entry = 1;
 		} else {
 			fprintf( stderr,
@@ -649,7 +618,7 @@ short_input:
 		/* Make sure all attributes with multiple values are contiguous */
 		for (; i<lines; i++) {
 			for (j=i+1; j<lines; j++) {
-				if ( !BVICMP( btype+i, btype+j )) {
+				if ( BV_CASEMATCH( btype+i, btype+j )) {
 					nmods--;
 					/* out of order, move intervening attributes down */
 					if ( j != i+1 ) {
@@ -683,13 +652,13 @@ short_input:
 		k = -1;
 		BER_BVZERO(&bv);
 		for (i=idn; i<lines; i++) {
-			if ( !BVICMP( btype+i, &BV_DN )) {
+			if ( BV_CASEMATCH( btype+i, &BV_DN )) {
 				fprintf( stderr, _("%s: attributeDescription \"%s\":"
 					" (possible missing newline"
 						" after line %d, entry \"%s\"?)\n"),
 					prog, btype[i].bv_val, linenum+i - 1, dn );
 			}
-			if ( BVICMP(btype+i,&bv)) {
+			if ( !BV_CASEMATCH( btype+i, &bv )) {
 				bvl[k++] = NULL;
 				bv = btype[i];
 				lm[j].mod_op = LDAP_MOD_ADD | LDAP_MOD_BVALUES;
@@ -728,11 +697,11 @@ short_input:
 
 			expect_modop = 0;
 			expect_sep = 1;
-			if ( BVICMP( btype+i, &BV_MODOPADD ) == 0 ) {
+			if ( BV_CASEMATCH( btype+i, &BV_MODOPADD )) {
 				modop = LDAP_MOD_ADD;
 				mops[i] = M_SEP;
 				nmods--;
-			} else if ( BVICMP( btype+i, &BV_MODOPREPLACE ) == 0 ) {
+			} else if ( BV_CASEMATCH( btype+i, &BV_MODOPREPLACE )) {
 			/* defer handling these since they might have no values.
 			 * Use the BVALUES flag to signal that these were
 			 * deferred. If values are provided later, this
@@ -741,11 +710,11 @@ short_input:
 				modop = LDAP_MOD_REPLACE;
 				mops[i] = modop | LDAP_MOD_BVALUES;
 				btype[i] = vals[i];
-			} else if ( BVICMP( btype+i, &BV_MODOPDELETE ) == 0 ) {
+			} else if ( BV_CASEMATCH( btype+i, &BV_MODOPDELETE )) {
 				modop = LDAP_MOD_DELETE;
 				mops[i] = modop | LDAP_MOD_BVALUES;
 				btype[i] = vals[i];
-			} else if ( BVICMP( btype+i, &BV_MODOPINCREMENT ) == 0 ) {
+			} else if ( BV_CASEMATCH( btype+i, &BV_MODOPINCREMENT )) {
 				modop = LDAP_MOD_INCREMENT;
 				mops[i] = M_SEP;
 				nmods--;
@@ -763,7 +732,7 @@ short_input:
 			expect_modop = 1;
 			nmods--;
 		} else {
-			if ( BVICMP( btype+i, &bv )) {
+			if ( !BV_CASEMATCH( btype+i, &bv )) {
 				fprintf( stderr, _("%s: wrong attributeType at"
 					" line %d, entry \"%s\"\n"),
 					prog, linenum+i, dn );
@@ -774,8 +743,9 @@ short_input:
 			/* If prev op was deferred and matches this type,
 			 * clear the flag
 			 */
-			if ( (mops[i-1]&LDAP_MOD_BVALUES) && !BVICMP(btype+i,
-				btype+i-1)) {
+			if ( (mops[i-1] & LDAP_MOD_BVALUES)
+				&& BV_CASEMATCH( btype+i, btype+i-1 ))
+			{
 				mops[i-1] = M_SEP;
 				nmods--;
 			}
@@ -790,7 +760,7 @@ short_input:
 		for (j=i+1; j<lines; j++) {
 			if ( mops[j] == M_SEP || mops[i] != mops[j] )
 				continue;
-			if ( !BVICMP( btype+i, btype+j )) {
+			if ( BV_CASEMATCH( btype+i, btype+j )) {
 				nmods--;
 				/* out of order, move intervening attributes down */
 				if ( j != i+1 ) {
@@ -836,7 +806,7 @@ short_input:
 	for (i=idn; i<lines; i++) {
 		if ( mops[i] == M_SEP )
 			continue;
-		if ( mops[i] != mops[i-1] || BVICMP(btype+i,&bv)) {
+		if ( mops[i] != mops[i-1] || !BV_CASEMATCH( btype+i, &bv )) {
 			bvl[k++] = NULL;
 			bv = btype[i];
 			lm[j].mod_op = mops[i] | LDAP_MOD_BVALUES;
@@ -1136,13 +1106,14 @@ domodify(
 			tool_perror( newentry ? "ldap_add" : "ldap_modify",
 				rc, NULL, NULL, NULL, NULL );
 			goto done;
-		} else if ( verbose ) {
-			printf( _("modify complete\n") );
 		}
-
 		rc = process_response( ld, msgid,
 			newentry ? LDAP_RES_ADD : LDAP_RES_MODIFY, dn );
 
+		if ( verbose && rc == LDAP_SUCCESS ) {
+			printf( _("modify complete\n") );
+		}
+
 	} else {
 		rc = LDAP_SUCCESS;
 	}
@@ -1168,12 +1139,12 @@ dodelete(
 			fprintf( stderr, _("%s: delete failed: %s\n"), prog, dn );
 			tool_perror( "ldap_delete", rc, NULL, NULL, NULL, NULL );
 			goto done;
-		} else if ( verbose ) {
-			printf( _("delete complete") );
 		}
-
 		rc = process_response( ld, msgid, LDAP_RES_DELETE, dn );
 
+		if ( verbose && rc == LDAP_SUCCESS ) {
+			printf( _("delete complete\n") );
+		}
 	} else {
 		rc = LDAP_SUCCESS;
 	}
@@ -1207,12 +1178,12 @@ dorename(
 			fprintf( stderr, _("%s: rename failed: %s\n"), prog, dn );
 			tool_perror( "ldap_rename", rc, NULL, NULL, NULL, NULL );
 			goto done;
-		} else {
-			printf( _("rename completed\n") );
 		}
-
 		rc = process_response( ld, msgid, LDAP_RES_RENAME, dn );
 
+		if ( verbose && rc == LDAP_SUCCESS ) {
+			printf( _("rename complete\n") );
+		}
 	} else {
 		rc = LDAP_SUCCESS;
 	}
