@@ -1,8 +1,8 @@
 /* csn.c - Change Sequence Number routines */
-/* $OpenLDAP: pkg/ldap/libraries/liblutil/csn.c,v 1.11.2.3 2006/01/03 22:16:11 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/liblutil/csn.c,v 1.14.2.4 2009/01/22 00:00:58 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2000-2006 The OpenLDAP Foundation.
+ * Copyright 2000-2009 The OpenLDAP Foundation.
  * Portions Copyright 2000-2003 Kurt D. Zeilenga.
  * All rights reserved.
  *
@@ -35,8 +35,8 @@
  * where s is a counter of operations within a timeslice, r is
  * the replica id (normally zero), and c is a counter of
  * modifications within this operation.  s, r, and c are
- * represented in hex and zero padded to lengths of 6, 2, and
- * 6, respectively.
+ * represented in hex and zero padded to lengths of 6, 3, and
+ * 6, respectively. (In previous implementations r was only 2 digits.)
  *
  * Calls to this routine MUST be serialized with other calls
  * to gmtime().
@@ -48,36 +48,19 @@
 
 #include <lutil.h>
 
+/* Must be mutex-protected, because lutil_gettime needs mutex protection */
 size_t
 lutil_csnstr(char *buf, size_t len, unsigned int replica, unsigned int mod)
 {
-	static time_t csntime;
-	static unsigned int csnop;
-
-	time_t t;
-	unsigned int op;
-	struct tm *ltm;
-#ifdef HAVE_GMTIME_R
-	struct tm ltm_buf;
-#endif
+	struct lutil_tm tm;
 	int n;
 
-	time( &t );
-	if ( t > csntime ) {
-		csntime = t;
-		csnop = 0;
-	}
-	op = csnop++;
+	lutil_gettime( &tm );
 
-#ifdef HAVE_GMTIME_R
-	ltm = gmtime_r( &t, &ltm_buf );
-#else
-	ltm = gmtime( &t );
-#endif
 	n = snprintf( buf, len,
-		"%4d%02d%02d%02d%02d%02dZ#%06x#%02x#%06x",
-	    ltm->tm_year + 1900, ltm->tm_mon + 1, ltm->tm_mday, ltm->tm_hour,
-	    ltm->tm_min, ltm->tm_sec, op, replica, mod );
+		"%4d%02d%02d%02d%02d%02d.%06dZ#%06x#%03x#%06x",
+	    tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday, tm.tm_hour,
+	    tm.tm_min, tm.tm_sec, tm.tm_usec, tm.tm_usub, replica, mod );
 
 	if( n < 0 ) return 0;
 	return ( (size_t) n < len ) ? n : 0;
