@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/libraries/liblutil/passfile.c,v 1.8.2.2 2007/08/31 23:13:57 quanah Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/liblutil/passfile.c,v 1.8.2.5 2009/03/09 23:16:48 quanah Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2007 The OpenLDAP Foundation.
+ * Copyright 1998-2009 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -62,7 +62,7 @@ lutil_get_filed_password(
 	}
 #endif /* HAVE_FSTAT */
 
-	passwd->bv_val = (char *) malloc( passwd->bv_len + 1 );
+	passwd->bv_val = (char *) ber_memalloc( passwd->bv_len + 1 );
 	if( passwd->bv_val == NULL ) {
 		perror( filename );
 		return -1;
@@ -73,10 +73,10 @@ lutil_get_filed_password(
 	do {
 		if( nleft == 0 ) {
 			/* double the buffer size */
-			char *p = (char *) realloc( passwd->bv_val,
+			char *p = (char *) ber_memrealloc( passwd->bv_val,
 				2 * passwd->bv_len + 1 );
 			if( p == NULL ) {
-				free( passwd->bv_val );
+				ber_memfree( passwd->bv_val );
 				passwd->bv_val = NULL;
 				passwd->bv_len = 0;
 				return -1;
@@ -89,7 +89,7 @@ lutil_get_filed_password(
 		nr = fread( &passwd->bv_val[nread], 1, nleft, f );
 
 		if( nr < nleft && ferror( f ) ) {
-			free( passwd->bv_val );
+			ber_memfree( passwd->bv_val );
 			passwd->bv_val = NULL;
 			passwd->bv_len = 0;
 			return -1;
