@@ -1,8 +1,8 @@
 /* config.h - configuration abstraction structure */
-/* $OpenLDAP: pkg/ldap/servers/slapd/config.h,v 1.34.2.4 2007/09/29 08:01:42 hyc Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/config.h,v 1.34.2.15 2009/06/02 23:41:33 quanah Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2007 The OpenLDAP Foundation.
+ * Copyright 1998-2009 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -19,15 +19,17 @@
 
 #include<ac/string.h>
 
+LDAP_BEGIN_DECL
+
 typedef struct ConfigTable {
-	char *name;
-	char *what;
+	const char *name;
+	const char *what;
 	int min_args;
 	int max_args;
 	int length;
 	unsigned int arg_type;
 	void *arg_item;
-	char *attribute;
+	const char *attribute;
 	AttributeDescription *ad;
 	void *notify;
 } ConfigTable;
@@ -56,6 +58,7 @@ typedef enum {
 #define ARG_BERVAL	0x00006000
 #define ARG_DN		0x00007000
 #define ARG_UINT	0x00008000
+#define ARG_ATDESC	0x00009000
 
 #define ARGS_SYNTAX	0xffff0000
 #define ARG_IGNORED	0x00080000
@@ -97,7 +100,7 @@ typedef int (ConfigCfAdd)(
 	Operation *op, SlapReply *rs, Entry *parent, struct config_args_s *ca );
 
 typedef struct ConfigOCs {
-	char *co_def;
+	const char *co_def;
 	ConfigType co_type;
 	ConfigTable *co_table;
 	ConfigLDAPadd *co_ldadd;
@@ -108,10 +111,10 @@ typedef struct ConfigOCs {
 
 typedef int (ConfigDriver)(struct config_args_s *c);
 
-typedef struct config_reply_s {
+struct config_reply_s {
 	int err;
 	char msg[SLAP_TEXT_BUFLEN];
-} ConfigReply;
+};
 
 typedef struct config_args_s {
 	int argc;
@@ -121,7 +124,7 @@ typedef struct config_args_s {
 	char *tline;
 	const char *fname;
 	int lineno;
-	char log[MAXPATHLEN + STRLENOF(": line 18446744073709551615") + 1];
+	char log[MAXPATHLEN + STRLENOF(": line ") + LDAP_PVT_INTTYPE_CHARS(unsigned long)];
 #define cr_msg reply.msg
 	ConfigReply reply;
 	int depth;
@@ -138,6 +141,7 @@ typedef struct config_args_s {
 			struct berval vdn_dn;
 			struct berval vdn_ndn;
 		} v_dn;
+		AttributeDescription *v_ad;
 	} values;
 	/* return values for emit mode */
 	BerVarray rvalue_vals;
@@ -150,7 +154,7 @@ typedef struct config_args_s {
 	BackendDB *be;
 	BackendInfo *bi;
 	Entry *ca_entry;	/* entry being modified */
-	void *private;	/* anything */
+	void *ca_private;	/* anything */
 	ConfigDriver *cleanup;
 	ConfigType table;	/* which config table did we come from */
 } ConfigArgs;
@@ -168,6 +172,9 @@ typedef struct config_args_s {
 #define value_bv values.v_bv
 #define value_dn values.v_dn.vdn_dn
 #define value_ndn values.v_dn.vdn_ndn
+#define value_ad values.v_ad
+
+int config_fp_parse_line(ConfigArgs *c);
 
 int config_register_schema(ConfigTable *ct, ConfigOCs *co);
 int config_del_vals(ConfigTable *cf, ConfigArgs *c);
@@ -196,4 +203,11 @@ int config_shadow( ConfigArgs *c, int flag );
 
 #define	SLAP_X_ORDERED_FMT	"{%d}"
 
+extern slap_verbmasks *slap_ldap_response_code;
+extern int slap_ldap_response_code_register( struct berval *bv, int err );
+
+extern ConfigTable olcDatabaseDummy[];
+
+LDAP_END_DECL
+
 #endif /* CONFIG_H */
